#include <cmath>
#include "point3d.h"

Point3D::Point3D(const double x, const double y, const double z) : _x{x}, _y{y}, _z{z}
{}

Point3D::Point3D(const std::vector<double> r) : _x{r[0]}, _y{r[1]}, _z{r[2]}
{}

void Point3D::translate(const double dx, const double dy, const double dz)
{
    _x += dx;
    _y += dy;
    _z += dz;
}

void Point3D::translate(const std::vector<double> dr)
{
    _x += dr[0];
    _y += dr[1];
    _z += dr[2];
}

void Point3D::rotate(const double alpha, const double beta, const double gamma)
{
    auto tx = _x * cos(beta) * cos(gamma) - _y * sin(gamma) * cos(beta) + _z * sin(beta);
    auto ty = _x * (sin(alpha) * sin(beta) * cos(gamma) + sin(gamma)*cos(alpha)) +
              _y * (-sin(alpha) * sin(beta) * sin(gamma) + cos(alpha) * cos(gamma)) -
              _z * sin(alpha) * cos(beta);
    auto tz = _x * (sin(alpha) * sin(gamma) - sin(beta) * cos(alpha) * cos(gamma)) +
              _y * (sin(alpha) * cos(gamma) + sin(beta) * cos(alpha) * sin(gamma)) +
              _z * cos(alpha) * cos(beta);
    _x = tx;
    _y = ty;
    _z = tz;
}


const std::vector<double> Point3D::coordinates()
{
    std::vector<double> result {_x, _y, _z};
    return result;
}


